#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "./regexp.h"

#define CODELEN 3

/* get_netcode:  a function that tests a filename to see if it looks like
                 a SAC filename generated by rdseed.  If so, returns the
                 network code from that file */

char *get_netcode(char *filename) {
  int test, len;
  char *out_phase, *tmp_ptr, *netcode;
  char pattern[1024] = "[0-9]+.[0-9]+.[0-9]+.[0-9]+.[0-9]+.[0-9]+.([A-Z0-9]+).[A-Z0-9]+.[A-Z0-9]+.SAC$";
  regexp *prog;

  if((prog = regcomp(pattern)) == NULL) {
    fprintf(stderr,"pattern '%s' didn't compile",pattern);
    return((char *)NULL);
  }

  if ((netcode = malloc(CODELEN*sizeof(char))) == (char *)NULL) {
    fprintf(stderr,"malloc() failed");
  }
  memset(netcode,0,CODELEN);
  tmp_ptr = filename;
  test = regexec(prog,tmp_ptr);

  if (prog->startp[0] != (char *)NULL && prog->startp[1] != (char *)NULL) {
    len = prog->endp[1] - prog->startp[1];
    strncpy(netcode,prog->startp[1], len);
  }
  else
    strcpy(netcode,"");

  return(netcode);
}

/* get_float:  a function that returns a floating point number from an input line
               using the input 'pattern' argument to set up a regular expression
               and parse the line.  A value of 1 is returned by the function if
               a match was found, otherwise a value of 0 is returned */

int get_float(char *input, char *pattern, float *value) {
  int test, len;
  char *tmp_ptr, *tmp_val;
  regexp *prog;

  if((prog = regcomp(pattern)) == NULL) {
    fprintf(stderr,"pattern '%s' didn't compile",pattern);
    return(0);
  }

  tmp_ptr = input;
  test = regexec(prog,tmp_ptr);

  if (prog->startp[0] != (char *)NULL && prog->startp[1] != (char *)NULL) {
    len = prog->endp[1] - prog->startp[1];
    if ((tmp_val = malloc((len+1)*sizeof(char))) == (char *)NULL) {
      fprintf(stderr,"get_float: malloc() failed");
    }
    strncpy(tmp_val,prog->startp[1], len);
    *value = (float)atof(tmp_val);
    free(tmp_val);
  }
  else
    return(0);

  return(1);
}

/* match_regexp:  tests an input string to see if it matches the regular
                  expression contained in the string 'pattern' */

int match_regexp(char *string, char *pattern) {
  int test, len;
  char *tmp_ptr;
  regexp *prog;

  if((prog = regcomp(pattern)) == NULL) {
    fprintf(stderr,"pattern '%s' didn't compile",pattern);
    return(0);
  }
  tmp_ptr = string;
  test = regexec(prog,tmp_ptr);
  return(test);
}
