#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "sac_rot.h"

#define SECLEN 8

void shift_dt(BTIME *t, double tshift) {
    double epoch_t;

    epoch_t = btime2epoch(t);
    epoch_t += tshift;
    epoch2btime(t, epoch_t);
}
/* fill the time structure t with values equivalent to epochal time epoch */

void epoch2btime(BTIME *t, double epoch) {
    int diy;
    double secleft;
    int sec, frac_sec;
    long days;
    char sec_str[SECLEN];

    days = (long) (epoch/86400.);
    secleft = fmod(epoch, 86400.0);
    t->hours = t->minutes= (UBYTE) 0;
    sec = 0.0;

    if(secleft) {               /* compute hours minutes seconds */
        if(secleft < 0) {       /* before 1970 */
            days--;             /* subtract a day */
            secleft+= 86400.;   /* add a day */
        }
        t->hours= (UBYTE) (secleft/3600.);
        secleft= fmod(secleft, 3600.0);
        t->minutes= (UBYTE) (secleft/60.);
        secleft = fmod(secleft, 60.0);
    }
    if (days >= 0){
        for (t->year = 1970; ; t->year++) {
            diy = is_leap(t->year) ? 366 : 365;
            if (days < diy) break;
            days -= diy;
        }
    }
    else {
        for (t->year= 1969; ; t->year--) {
            diy = is_leap(t->year) ? 366 : 365;
            days += diy;
            if (days >= 0) break;
        }
    }
    days++;
    t->day = (UBYTE2) days;
    memset(sec_str,0,SECLEN);
    sprintf(sec_str,"%07.4f",secleft);
    sscanf(sec_str,"%d.%d", &sec, &frac_sec);
    t->seconds = (UBYTE) sec;
    t->frac_secs = (UBYTE2) frac_sec;
    return;
}

/* calculate epochal equivalent of time t */

double btime2epoch(BTIME *t) {
    UBYTE2 i;
    long days;
    double epoch, sec;
    char sec_str[SECLEN];

    days = 0;
    memset(sec_str,0,SECLEN);
    sprintf(sec_str,"%2.2d.%4.4d",t->seconds,t->frac_secs);
    sec = atof(sec_str);
    if (t->year > 1970){
        for (i = (UBYTE2) 1970; i < t->year; i++) {
            days += 365;
            if (is_leap(i))  days++;
        }
    }
    else if (t->year < 1970) {
        for (i = t->year; i < (UBYTE2) 1970; i++) {
            days -= 365;
            if (is_leap(i)) days--;
        }
    }
    days += (long) (t->day-(UBYTE2)1);
    epoch = (((double) days)*86400.);
    epoch += ((double)t->hours*3600.)+((double)t->minutes*60.)+sec;
    return(epoch);
}

/* return true if leap year else false */

int is_leap(UBYTE2 year) {
    return(year % 4 == 0 && year % 100 != 0 || year % 400 == 0);
}
