#include <math.h>

#define ABS(a) ((a) > 0 ? (a) : -(a))

/* get_mean:  determines the mean value of an input floating point array */

float get_mean(float *data_ptr, int npts) {
    int i;
    float mean_val = 0.0;

    for(i = 0; i < npts; i++) {
        mean_val += *(data_ptr+i)/(float)npts;
    }
    return(mean_val);
}

/* taper:  applies a cosine taper to the data */

void taper(float *data_ptr, int npts, float fraction) {
    float wj, win_width;
    int j;

    win_width = (float)npts*fraction/2.0;
    for(j = 0; j < (int)win_width; j++) {
        wj = (float) (1.0 - pow((((double)j-win_width)/win_width),2.0));
        *(data_ptr+j) *= wj;
        if(j < (npts-j))
            *(data_ptr+npts-j) *= wj;
    }
}

/* find_north:  returns the index of the argument that is the 'north' component (versus
                the 'east' component.  Note:  the 'north' component is the component
                that moves closer to the 'east' component when its azimuth is increased) */

int find_north(int cmp1az, int cmp2az) {
    float a, b;
    float abs_angle;

    a = cmp1az;
    b = cmp2az;
    abs_angle = ABS(a-b);
    if(abs_angle > 180.0) {
        if(a > b) {
            return(1);
        }
        else if(b > a) {
            return(2);
        }
        else
            return(0);
    }
    else if(abs_angle < 180.0 && abs_angle > 0.0) {
        if(a > b) {
            return(2);
        }
        else if(b > a) {
            return(1);
        }
        else
            return(0);
    }
    else
       return(0);
}
